/*
 * An emulator, visualiser of the steps of an arrangement of LLFSMs from the output
 * a trace on nuXmv or NuSMV
 * (c) 2020 Vlad Estivill-Castro
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see www.gnu.org/licenses.
 * @see<a href="https://www.gnu.org/licenses"> https://www.gnu.org/licenses </a>.
 */
package emulatenuxmvtracesofllfsms;

import java.util.Hashtable;
import java.util.Map;
import java.util.HashMap;
import java.util.ArrayList;
import java.awt.BorderLayout;
import java.util.Enumeration;

import com.mxgraph.layout.hierarchical.mxHierarchicalLayout;
import com.mxgraph.util.mxConstants;
import com.mxgraph.view.mxGraph;
import com.mxgraph.model.mxCell;
import com.mxgraph.layout.mxGraphLayout;
import com.mxgraph.swing.mxGraphComponent;
import com.mxgraph.swing.util.mxMorphing;
import com.mxgraph.util.mxEvent;
import com.mxgraph.util.mxEventObject;
import com.mxgraph.util.mxEventSource;
import com.mxgraph.view.mxStylesheet;

/**
 *
 * @author vlad estivill-castro
 */
public class VisualPanelForMachine extends javax.swing.JPanel  {
    
    public EmulatenuXmvTracesOfLLFSms mainWin;
    private  mxGraph graph;
    private  mxGraphComponent graphComponent;
    private  mxGraphLayout layout;
    public Hashtable<String, Object> states;
    private final Hashtable<String, Object> transitions;
    public Hashtable<String, Object> transition_strings;
    private final String lightGreen="#66ff33";
    private final String lightBlue="#66FFFF";
    private final String darkRed="#cc0000";
    private final String darkBlue="#000066";
    
    private void initComponents() {

        javax.swing.GroupLayout aLayout = new javax.swing.GroupLayout(this);
        this.setLayout(aLayout);
        aLayout.setHorizontalGroup(
            aLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 400, Short.MAX_VALUE)
        );
        aLayout.setVerticalGroup(
            aLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 300, Short.MAX_VALUE)
        );
    }
    
     /**
     * Constructor creates new form VisualPanel
     */
public VisualPanelForMachine() {
        initComponents();

        graph = new mxGraph();
        states = new Hashtable<>();
        transitions = new Hashtable<>();
        transition_strings = new Hashtable<>();

        layout = new mxHierarchicalLayout(graph);

        Map<String, Object> edgeStyle = new HashMap<String, Object>();
        //edgeStyle.put(mxConstants.STYLE_EDGE, mxConstants.EDGESTYLE_ORTHOGONAL);
        edgeStyle.put(mxConstants.STYLE_SHAPE,    mxConstants.SHAPE_CONNECTOR);
        edgeStyle.put(mxConstants.STYLE_ENDARROW, mxConstants.ARROW_CLASSIC);
        edgeStyle.put(mxConstants.STYLE_STROKECOLOR, "#000000");
        edgeStyle.put(mxConstants.STYLE_FONTCOLOR, "#000000");
        edgeStyle.put(mxConstants.STYLE_LABEL_BACKGROUNDCOLOR, "#ffffff");

        mxStylesheet stylesheet = new mxStylesheet();
        stylesheet.setDefaultEdgeStyle(edgeStyle);

        graph.setStylesheet(stylesheet);

        graph.setCellsEditable(false);
        graph.setCellsResizable(false);
        graph.setCellsDisconnectable(false);

        graphComponent = new mxGraphComponent(graph);


        //graphComponent = null;
        graphComponent.setConnectable(false);

        setLayout(new BorderLayout()); 
        
        add(graphComponent);
        
} /* constructor */
        
    public void HighlightState(String name, String theColorString) {
        graph.getModel().beginUpdate();
        System.err.println("Highlighting  state"+name);
        Object s1 = states.get(name);
        mxCell theCell = (mxCell) s1;
        String style = graph.getModel().getStyle(theCell);

        graph.setCellStyles(mxConstants.STYLE_FILLCOLOR, lightBlue, new Object[]{theCell});
        graph.getModel().endUpdate();
    }
       
    /**
     * @brief Add a transition to the graph
     * @param state1 The originating state
     * @param state2 The destination state
     * @param expression The transition condition
     */
    public void AddTransition(String state1, String state2, String expression) {
        Object s1 = states.get(state1);
        Object s2 = states.get(state2);

        if ( (null==s1)|| (null==s2) ) {
                System.err.println("BAD state");
        }

        graph.getModel().beginUpdate();

        Object e = graph.insertEdge(graph.getDefaultParent(), null, expression, s1, s2);

        transitions.put(state1+state2+expression, e);
        transition_strings.put(state1 + "->" + state2 + ":" + expression, e);

        graph.getModel().endUpdate();
    }   

    /**
     * @breif Add a state to the graph
     * @param name The name of the state to add
     */
    public void AddState(String name, String theStatements) {
        AddState(name, 0.0f, 0.0f,theStatements);
    }

    /**
     * @breif Add a state to the graph
     * @param name The name of the state to add
     */
    public void AddState(String name, float x, float y, String theStatements ) {

        graph.getModel().beginUpdate();

        // keep the key the state name and id, but display the statements as well
        Object s = graph.insertVertex(graph.getDefaultParent(), null, name+"\n"+theStatements, x, y, 130, 80,"defaultVertex;fillColor=green");

        states.put(name, s);
        System.err.println("Adding state");

        HighlightState (name,lightGreen);

        graph.getModel().endUpdate();
    }    // AddState
        
    public void SetPCState(String thePCofState) {
        graph.getModel().beginUpdate();
        Enumeration<String> enumeration = states.keys();

        while(enumeration.hasMoreElements()) {
                String name = enumeration.nextElement();
                Object s1 = states.get(name);
                mxCell theCell = (mxCell) s1;
            String style = graph.getModel().getStyle(theCell);
            if (name.contains("("+thePCofState+")"  )) {
                graph.setCellStyles(mxConstants.STYLE_FILLCOLOR, lightGreen, new Object[]{theCell});
            } else
                graph.setCellStyles(mxConstants.STYLE_FILLCOLOR, lightBlue, new Object[]{theCell});

        }

        graph.getModel().endUpdate();
    }// SetPCState
    
    
    public void highlightTransition(String thetransitionIDString) {
        graph.getModel().beginUpdate();
        Enumeration<String> enumeration = transitions.keys();
        while(enumeration.hasMoreElements()) {
                String theTransitionLabel = enumeration.nextElement();
                Object s1 = transitions.get(theTransitionLabel);
                mxCell theCell = (mxCell) s1;
            String style = graph.getModel().getStyle(theCell);
            if (theTransitionLabel.contains(thetransitionIDString  )) {
                graph.setCellStyles(mxConstants.STYLE_FONTCOLOR, darkRed, new Object[]{theCell});
            } else
                graph.setCellStyles(mxConstants.STYLE_FONTCOLOR, darkBlue, new Object[]{theCell});
        } //while
        graph.getModel().endUpdate();
    } // highlightTransition
    
    
    public void regularTransitionColor() {
        graph.getModel().beginUpdate();
        Enumeration<String> enumeration = transitions.keys();
        while(enumeration.hasMoreElements()) {
                String theTransitionLabel = enumeration.nextElement();
                Object s1 = transitions.get(theTransitionLabel);
                mxCell theCell = (mxCell) s1;
                String style = graph.getModel().getStyle(theCell);
                graph.setCellStyles(mxConstants.STYLE_FONTCOLOR, darkBlue, new Object[]{theCell});

        }
        graph.getModel().endUpdate();
    }
   
    /**
     * @brief Apply an automatic layout to the graph
     */
    public void AutoLayout() {
        //System.out.println("Layout");
        layout.execute(graph.getDefaultParent());

        mxMorphing morph = new mxMorphing(graphComponent, 20, 1.2, 20);

        morph.addListener(mxEvent.DONE, new mxEventSource.mxIEventListener() {

                @Override
                public void invoke(Object arg0, mxEventObject arg1) {
                    graph.getModel().endUpdate();
                    // fitViewport();
                }

            });

        morph.startAnimation();
    }
    
} /* end VisualPanelForMachine */
